@@ -45,14 +45,37 @@ randi <- function(imax, n=1, m=n) { # drop?
 }
 
 
+rands <- function (n = 1, N = 1, r = 1) 
+{
+    if (n < 1 || N < 1 || r < 0) return(c())
+    X <- randn(n, N+1)
+    Y <- sqrt(rowSums(X^2))
+    return(r * X/Y)
+}
+
+
 randp <- function(n = 1, r = 1) {
     if (n < 1 || r < 0) return(c())
-    x <- rnorm(n)
-    y <- rnorm(n)
+    x <- rnorm(n); y <- rnorm(n)
     r <- r * sqrt(runif(n)/(x^2 + y^2))
-    if (n == 1)
-        U <- c(x, y)
-    else
-        U <- cbind(r*x, r*y)
-    return(U)
+    return(cbind(r*x, r*y))
 }
+
+
+randsample <- function(n, k, w = NULL, replacement = FALSE) {
+	stopifnot(is.numeric(n), is.numeric(k))
+	if (length(n) == 1) n <- 1:floor(n)
+	else                n <- c(n)
+	if (k > length(n) && !replacement) {
+		warning("k > n or length(n): replacement will be set to TRUE.")
+		replacement = TRUE
+	}
+	if (is.numeric(w)) {
+		if (!replacement) replacement = TRUE
+		if (length(n) != length(w))
+			stop("Weights vector 'w' must have the same length as 'n'.")
+	}
+
+	sample(n, k, replace = replacement, prob = w)
+}
+
